/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.def.majorElement;

import java.io.Serializable;
import java.util.Set;

import org.ow2.bonita.facade.def.element.DeadlineDefinition;
import org.ow2.bonita.facade.def.element.HookDefinition;
import org.ow2.bonita.facade.def.element.MultiInstantiationDefinition;
import org.ow2.bonita.facade.def.element.PerformerAssignDefinition;
import org.ow2.bonita.facade.def.element.SimulationInformationDefinition;
import org.ow2.bonita.facade.def.element.SubFlowDefinition;
import org.ow2.bonita.facade.def.element.ToolDefinition;
import org.ow2.bonita.facade.def.element.TransitionRestrictionDefinition;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;

/**
 * This interface represents the Activity definition.
 * It's derived from the Process Activity Definition of XPDL.
 *
 */
public interface ActivityDefinition extends ProcessDefinitionRecord, Serializable {

  /**
   * StartMode parameter determines whether the activity is automatically started by the engine or not.
   */
  public static enum StartMode {
    Automatic, Manual
  }

  /**
   * Not supported (because XPDL is not clear enough for this parameter).
   */
  public static enum FinishMode {
    Automatic, Manual
  }

  /**
   * Interface for Iteration definition.
   */
  public static interface IterationDef {
    String getTo();
    String getCondition();
  }
  /**
   * Returns the UUID for the ActivityDefinition.
   */
  ActivityDefinitionUUID getUUID();
  /**
   * Returns the value of the id attribute of the Activity element defined into the XPDL file.
   */
  String getActivityId();

  /**
   * Short textual description of the activity.
   */
  String getDescription();
  /**
   * Expected duration for time management purposes (e.g. starting an escalation procedure etc.)
   * inunits of DurationUnit. It is counted from the starting date/time of the Process.
   * The consequences of reaching the limit value is depending on the engine.<br>
   * This is not yet supported.
   */
  String getLimit();

  //activity type
  /**
   * Activity can be defined either with Route or Implementation or BlockActivity element.
   * @return true if the activity has been defined as a Route activity otherwise returns false.
   *
   */
  boolean isRoute();
  /**
   * Activity can be defined either with Route or Implementation or BlockActivity element.
   * In case activity has been defined with an Implementation element, its child element
   * could be either No or Tool or SubFlow element.
   * @return true if the No element is defined within the Implementation element
   * otherwise false.
   *
   */
  boolean isNoImplementation();
  /**
   * Returns the SubFlowDefinition interface if this activity has subflow implementation
   * otherwise returns null.
   */
  SubFlowDefinition getSubFlow();
  /**
   * Returns (unordered) set of ToolDefinition interfaces.
   * Returns an empty set if no tools implementations are defined.
   */
  Set<ToolDefinition> getTools();
  /**
   * Not supported.
   */
  String getBlockId();
  /**
   * Returns the performer of the activity.
   * For an activity with startMode=Manual the performer element contains a defined
   * participant processDefinitionUUID.
   * For an activity with startMode=Automatic the performer element contains the
   * generic participant SYSTEM.
   */
  String getPerformer();
  /**
   * If StartMode=Manual a task entity is created by the engine and stored in the
   * task repository and is waiting to be take in account by an end user.<br>
   * If StartMode=Automatic no task is created. the activity is automatically performed
   * @return The {@link StartMode StartMode enumeration} value.
   */
  StartMode getStartMode();
  /**
   * Not supported.
   */
  FinishMode getFinishMode();
  /**
   * A value that describes the initial priority of this activity when it starts execution.
   * If this attribute is not defined but a priority is defined in the Process definition
   * then that is used.<br>
   * By default it is assumed that the priority levels are the natural numbers starting
   * with zero, and that the higher the value the higher the priority (i.e.: 0,...., n).<br>
   * Not yet supported.
   */
  String getPriority();
  /**
   * Deadlines are used to execute a specified java class upon the expiration of
   * a specified period of time.
   * returns (unordered) set of Deadlines.
   * returns empty set if no deadline are defined within the activity.
   */
  Set<DeadlineDefinition> getDeadlines();
  /**
   * Estimations for simulation of an Activity. No default.
   * Not supported.
   */
  SimulationInformationDefinition getSimulationInformation();
  /**
   * Address (path- and filename) for an icon to represent the activity in a graphical modeller.
   * May be used to override the modeller icon for an activity.
   */
  String getIcon();
  /**
   * The address (e.g. path- and filename) for a help file or a description file of the activity.
   */
  String getDocumentation();
  /**
   * Gives the routing behavior if outgoing transitions
   * or/and incoming transitions are defined.
   * Two types of transition element exist: SPLIT and JOIN.
   * @return The (unordered) set of TransitionRestrictionDefinition interfaces.
   */
  Set<TransitionRestrictionDefinition> getTransitionRestrictions();
  /**
   * Bonita engine allows to specify datafields for activities.
   * As xpdl v1 does not allow this definition, extended attributes are specified
   * for this purpose.
   * @return The (unordered) set of DataFieldDefinition interfaces.
   */
  Set<DataFieldDefinition> getDataFields();
  /**
   * Hooks are user defined logic that can be triggered at some points of the life of
   * the activity.<br>
   * Those points are:
   * <ul>
   * <li>taskOnReady</li>
   * <li>taskOnStart</li>
   * <li>taskOnFinish</li>
   * </ul>
   * <br>
   * If the activity has startMode=Manual then these points of the life of the activity
   * is synchronized with ones of the task.
   * @return The (unordered) set of HookDefinition interfaces.
   */
  Set<HookDefinition> getHooks();
  /**
   * Performer assignment allows to perform various assignment rules within the task module.<br>
   * All the users defined into the role type can see and execute this one.
   * By adding this functionality, we can:
   * <br>
   * <ul>
   * <li>assign the task to a user of the role (group) by calling a java class in charge to the user selection
   * into the user group (callback performer assignment)</li>
   * <li>assign dynamically the activity to a user by using an activity variable</li>
   * (property performer assignment)</li>
   * </ul>
   * @return The PerformerAssignDefinition interface.
   */
  PerformerAssignDefinition getPerformerAssign();

  Set<IterationDef> getIterations();

  MultiInstantiationDefinition getMultiInstantiationDefinition();
  
  boolean isAsynchronous();
}


